/* joystick emulation
 *
 * (C) 2018 by Andreas Eversberg <jolly@eversberg.eu>
 * All Rights Reserved
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include "joystick.h"

static int joystick_left = 0;
static int joystick_right = 0;
static int joystick_up = 0;
static int joystick_down = 0;
static int joystick_fire = 0;

#define CIAAPRA		0xbfe000
#define JOYDAT1		0xdff00c

uint16_t emulate_joystick_read(uint32_t address)
{
	if (address == CIAAPRA) {
		if (joystick_fire)
			return 0xff7f;
		return 0xffff;
	}

	if (address == JOYDAT1) {
		uint16_t joydat = 0x0000;
		if (joystick_right) {
			joydat |= 0x0002;
			/* make bit 0 and 1 equal */
			if (!joystick_down)
				joydat |= 0x0001;
		} else {
			/* make bit 0 and 1 odd */
			if (joystick_down)
				joydat |= 0x0001;
		}
		if (joystick_left) {
			joydat |= 0x0200;
			/* make bit 8 and 9 equal */
			if (!joystick_up)
				joydat |= 0x0100;
		} else {
			/* make bit 8 and 9 odd */
			if (joystick_up)
				joydat |= 0x0100;
		}
		return joydat;
	}

	return 0xffff;
}

/* use -1 to keep state unchanged */
void set_joystick(int left, int right, int up, int down, int fire)
{
	if (left >= 0)
		joystick_left = left;
	if (right >= 0)
		joystick_right = right;
	if (up >= 0)
		joystick_up = up;
	if (down >= 0)
		joystick_down = down;
	if (fire >= 0)
		joystick_fire = fire;
}


