/* generates hex code from game binary
 *
 * (C) 2018 by Andreas Eversberg <jolly@eversberg.eu>
 * All Rights Reserved
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>

int main(int argc, char *argv[])
{
	FILE *ifp, *ofp;
	int a, b, c, d;
	int n = 0;

	if (argc <= 3) {
		fprintf(stderr, "Usage: %s <binary> <souce code> <array name>\n", argv[0]);
		return -1;
	}

	ifp = fopen(argv[1], "r");
	if (!ifp) {
		fprintf(stderr, "Failed to open input file: %s\n", argv[1]);
		return -1;
	}
	ofp = fopen(argv[2], "w");
	if (!ofp) {
		fprintf(stderr, "Failed to open output file: %s\n", argv[2]);
		return -1;
	}

	fprintf(ofp, "\n#include <stdint.h>\n");
	fprintf(ofp, "\n/* Data is stored as 'big endian' and must be converted to host machine's endianess. */\n");
	fprintf(ofp, "const uint32_t %s[] = {", argv[3]);
	while ((a = fgetc(ifp)) != EOF) {
		b = fgetc(ifp);
		c = fgetc(ifp);
		d = fgetc(ifp);
		if (b == EOF || c == EOF || d == EOF) {
			fprintf(stderr, "Short read of input file. Input file's size must be a multiple of 4!\n");
			return -1;
		}
		if ((n % 32) == 0)
			fprintf(ofp, "\n\t");
		fprintf(ofp, "0x%02x%02x%02x%02x,", a, b, c, d);
		n += 4;
	}
	fprintf(ofp, "\n};\n\n");
	fprintf(ofp, "int %s_size = %d;\n\n", argv[3], n);

	fclose(ifp);
	fclose(ofp);
}

