/* game specials
 *
 * (C) 2018 by Andreas Eversberg <jolly@eversberg.eu>
 * All Rights Reserved
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include "../libcpu/m68k.h"
#include "../libcpu/execute.h"
#include "mercenary.h"

#define	INITIAL_STACK	0x7fffa
#define RESET_VECTOR	0x5a16c
// #define RESET_VECTOR	0x5a1a4 /* strange reset vector that causes the player to fly around */

/* interrupt CPU execution at special break points and tell emulation what to do */
const struct cpu_stop mercenary_stop_at[] = {
	{ 0x5a826,	STOP_AT_WAIT_VBL },	/* done with rendering, waiting for VBL */
	{ 0x55d8c,	STOP_AT_WAIT_VBL },	/* after pressing 'HELP' key before showing menu line on benson */
	{ 0x56398,	STOP_AT_WAIT_VBL },	/* waiting for menu command */
	{ 0x55d94,	STOP_AT_WAIT_VBL },	/* after pressing 'HELP' key while showing menu line on benson */
	{ 0x563a6,	STOP_AT_WAIT_VBL },	/* after pressing 'RETURN' while game waits for other key to resume */
	{ 0x52946,	STOP_AT_WAIT_VBL },	/* after dying, waiting for VBL to fade out palette */
	{ 0x0,		STOP_AT_END },		/* end */
};

extern const uint32_t mercenary3_hex[];
extern int mercenary3_hex_size;

void mercenary_load(void)
{
	int i;

	/* load game binary from constant to volatile memory */
	for (i = 0; i < mercenary3_hex_size; i += 4) {
		m68k_write_memory_32(i, mercenary3_hex[i / 4]);
	}
}

void mercenary_patch(void)
{
	/* initial stack */
	m68k_write_memory_32(0x00000, INITIAL_STACK);

	/* reset vector */
	m68k_write_memory_32(0x00004, RESET_VECTOR);

	/* patch away the function call that is outside chip memory
	 * is this a copy protection????
	 */
	m68k_write_memory_16(0x5a17c, 0x4e71); /* nop */
	m68k_write_memory_16(0x5a17e, 0x4e71); /* nop */
	m68k_write_memory_16(0x5a180, 0x4e71); /* nop */

	/* remove wait for VBL */
	m68k_write_memory_16(0x5a82C, 0x4e71); /* nop */

	/* reduce loop that waits for disk stepper to move */
	if (m68k_read_memory_32(0x562f8) != 0x000091b0) {
		fprintf(stderr, "expecting loop counter of 0x000091b0 here, please fix!\n");
		exit(0);
	}
	m68k_write_memory_32(0x562f8, 1);
	/* reduce loop that waits for disk side change */
	if (m68k_read_memory_32(0x55f5c) != 0x0000d020) {
		fprintf(stderr, "expecting loop counter of 0x0000d020 here, please fix!\n");
		exit(0);
	}
	m68k_write_memory_32(0x55f5c, 1);
}

uint32_t mercenary_palette(void)
{
	return m68k_read_memory_32(0x0072b0);
}

